/*
 * @Author: Jeepeng 
 * @Date: 2020-02-23 02:09:45 
 * @Last Modified by: Jeepeng
 * @Last Modified time: 2020-03-03 19:14:29
 */
package com.jeepeng.reactnative.geolocation;

import androidx.annotation.Nullable;

import com.amap.api.location.AMapLocation;
import com.amap.api.location.AMapLocationClient;
import com.amap.api.location.AMapLocationClientOption;
import com.amap.api.location.AMapLocationListener;
import com.amap.api.location.CoordinateConverter;
import com.amap.api.location.DPoint;
import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.LifecycleEventListener;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.Callback;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.modules.core.DeviceEventManagerModule;

public class RNGeoLocationAMapModule extends ReactContextBaseJavaModule implements LifecycleEventListener, AMapLocationListener {

    private final ReactApplicationContext reactContext;

    // 声明AMapLocationClient类对象
    private AMapLocationClient mLocationClient = null;
    // 声明AMapLocationClientOption对象
    private AMapLocationClientOption mLocationOption = null;

    public RNGeoLocationAMapModule(ReactApplicationContext reactContext) {
        super(reactContext);
        this.reactContext = reactContext;
        // 初始化
        mLocationOption = new AMapLocationClientOption();
        mLocationClient = new AMapLocationClient(reactContext);
        mLocationClient.setLocationListener(this);
    }

    @ReactMethod
    public void init(String key, Promise promise) {
        // setKey必须在AmapLocationClient实例化之前调用
        if (mLocationClient != null) {
            mLocationClient.onDestroy();
        }
        AMapLocationClient.setApiKey(key);
        mLocationClient = new AMapLocationClient(reactContext);
        mLocationClient.setLocationListener(this);
        promise.resolve(null);
    }

    @Override
    public String getName() {
        return "RNGeoLocationAMap";
    }

    private void sendEvent(String eventName, @Nullable WritableMap params) {
        this.reactContext.getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class).emit(eventName, params);
    }

    @ReactMethod
    public void startLocation() {
        // 启动定位
        mLocationClient.setLocationOption(this.mLocationOption);
        mLocationClient.startLocation();
    }

    @ReactMethod
    public void stopLocation() {
        // 停止定位后，本地定位服务并不会被销毁
        mLocationClient.stopLocation();
    }

    @ReactMethod
    public void setLocationCacheEnable(Boolean bool) {
        // 关闭缓存机制
        mLocationOption.setLocationCacheEnable(bool);
    }

    /**
     * 设置定位场景
     */
    @ReactMethod
    public void setLocationPurpose(String purpose) {
        mLocationOption.setLocationPurpose(AMapLocationClientOption.AMapLocationPurpose.valueOf(purpose));
    }

    @ReactMethod
    public void setGeoLanguage(String lang) {
        mLocationOption.setGeoLanguage(AMapLocationClientOption.GeoLanguage.valueOf(lang));
    }

    @ReactMethod
    public void setOnceLocationLatest(Boolean bool) {
        mLocationOption.setOnceLocationLatest(bool);
    }

    @ReactMethod
    public void setWifiScan(boolean isWifiScan) {
        mLocationOption.setWifiScan(isWifiScan);
    }

    @ReactMethod
    public void calculateLineDistance(double latitude1, double longitude1, double latitude2, double longitude2, Promise promise) {
        DPoint p1 = new DPoint(latitude1, longitude1);
        DPoint p2 = new DPoint(latitude2, longitude2);
        double distance = CoordinateConverter.calculateLineDistance(p1, p2);
        promise.resolve(distance);
    }

    @Override
    public void onLocationChanged(AMapLocation amapLocation) {
        WritableMap params = Arguments.createMap();
        if (null != amapLocation) {
            // errCode等于0代表定位成功，其他的为定位失败，具体的可以参照官网定位错误码说明
            if (amapLocation.getErrorCode() == 0) {
                params.putInt("errorCode", amapLocation.getErrorCode());
                // 获取当前定位结果来源，如网络定位结果，详见定位类型表
                params.putInt("locationType", amapLocation.getLocationType());
                // 获取纬度
                params.putDouble("latitude", amapLocation.getLatitude());
                // 获取经度
                params.putDouble("longitude", amapLocation.getLongitude());
                // 海拔
                params.putDouble("altitude", amapLocation.getAltitude());
                // 获取精度(米)
                params.putDouble("accuracy", amapLocation.getAccuracy());
                // 获取提供者
                params.putString("provider", amapLocation.getProvider());
                // 地址，如果option中设置isNeedAddress为false，则没有此结果，网络定位结果中会有地址信息，GPS定位不返回地址信息。
                params.putString("address", amapLocation.getAddress());
                // 国家
                params.putString("country", amapLocation.getCountry());
                // 省
                params.putString("province", amapLocation.getProvince());
                // 城市
                params.putString("city", amapLocation.getCity());
                // 区
                params.putString("district", amapLocation.getDistrict());
                // 街道
                params.putString("street", amapLocation.getStreet());
                // 街道门牌号
                params.putString("streetNum", amapLocation.getStreetNum());
                // 城市编码
                params.putString("cityCode", amapLocation.getCityCode());
                // 地区编码
                params.putString("adCode", amapLocation.getAdCode());
                // 获取当前定位点的AOI信息
                params.putString("aoiName", amapLocation.getAoiName());
                // 兴趣点
                params.putString("poiName", amapLocation.getPoiName());
                // 当前室内定位的建筑物Id
                params.putString("buildingId", amapLocation.getBuildingId());
                // 当前室内定位的楼层
                params.putString("floor", amapLocation.getFloor());
                // GPS的当前状态
                params.putInt("GpsAccuracyStatus", amapLocation.getGpsAccuracyStatus());
                // 速度(米/秒)
                params.putDouble("speed", amapLocation.getSpeed());
                // 角度
                params.putDouble("bearing", amapLocation.getBearing());
                // 角度, 同 bearing
                params.putDouble("hearing", amapLocation.getBearing());
                // 获取当前提供定位服务的卫星个数
                params.putInt("satellites", amapLocation.getSatellites());
                // 定位时间
                params.putDouble("timestamp", amapLocation.getTime());
            } else {
                // 错误码
                params.putInt("errorCode", amapLocation.getErrorCode());
                // 错误信息
                params.putString("errorInfo", amapLocation.getErrorInfo());
                // 错误描述
                params.putString("locationDetail", amapLocation.getLocationDetail());
            }
            // WIFI开关
            params.putBoolean("isWifiAble", amapLocation.getLocationQualityReport().isWifiAble());
            // GPS状态
            params.putInt("GPSStatus", amapLocation.getLocationQualityReport().getGPSStatus());
            // GPS星数
            params.putInt("GPSSatellites", amapLocation.getLocationQualityReport().getGPSSatellites());
        } else {
            // 错误码
            params.putInt("errorCode", -1);
            // 错误信息
            params.putString("errorInfo", "定位失败，amapLocation is null");
        }
        sendEvent("onLocationChanged", params);
    }

    @Override
    public void onHostResume() {

    }

    @Override
    public void onHostPause() {

    }

    @Override
    public void onHostDestroy() {
        // 销毁定位客户端，同时销毁本地定位服务。
        if (null != mLocationClient) {
            mLocationClient.onDestroy();
            mLocationClient = null;
            mLocationOption = null;
        }
    }
}
