/*
 * @Author: Jeepeng 
 * @Date: 2020-02-22 15:41:30 
 * @Last Modified by: Jeepeng
 * @Last Modified time: 2020-03-03 19:33:41
 */

import { Platform, NativeModules, NativeEventEmitter } from 'react-native';

const { RNGeoLocationAMap } = NativeModules;
const GeoLocationAMapEventEmitter = new NativeEventEmitter(RNGeoLocationAMap);

if (!RNGeoLocationAMap) {
  throw new Error(`react-native-geolocation-amap: NativeModules.RNGeoLocationAMap is null. To fix this issue try these steps:
• Run \`react-native link react-native-geolocation-amap\` in the project root.
• Rebuild and re-run the app.
• If you are using CocoaPods on iOS, run \`pod install\` in the \`ios\` directory and then rebuild and re-run the app. You may also need to re-open Xcode to get the new pods.`);
}

let watchId = 0;

const locationChangedHandler = (result, success, error) => {
  if (result.errorCode === 0) {
    success && success({
      ...result,
      coords: {
        accuracy: result.accuracy,
        latitude: result.latitude,
        longitude: result.longitude,
        altitude: result.altitude,
        speed: result.speed,
        heading: result.heading,
        altitudeAccuracy: null,
      },
    });
  } else {
    error && error({ code: result.errorCode, message: result.errorInfo });
  }
};

class GeoLocationAMap {
  static _subscriptions = new Map();
  static init(apiKeyMap = {}) {
    RNGeoLocationAMap.init(Platform.select(apiKeyMap));
    if (Platform.OS === 'ios') {
      RNGeoLocationAMap.setLocatingWithReGeocode(true); // 默认返回地址信息
    }
  }
  static start() {
    if (Platform.OS === 'ios') {
      RNGeoLocationAMap.startUpdatingLocation();
    } else {
      RNGeoLocationAMap.startLocation();
    }
  }

  static stop() {
    if (Platform.OS === 'ios') {
      RNGeoLocationAMap.stopUpdatingLocation();
    } else {
      RNGeoLocationAMap.stopLocation();
    }
  }

  static setLocationPurpose(purpose) {
    if (Platform.OS === 'android') {
      RNGeoLocationAMap.setLocationPurpose(GeoLocationAMapPurpose[purpose] || GeoLocationAMapPurpose.SignIn);
    }
  }

  static setGeoLanguage(lang) {
    if (Platform.OS === 'android') {
      RNGeoLocationAMap.setGeoLanguage(GeoLocationLanguage[lang] || GeoLocationLanguage.DEFAULT);
    }
  }

  static setLocationCacheEnable(isEnable) {
    if (Platform.OS === 'android') {
      RNGeoLocationAMap.setLocationCacheEnable(isEnable);
    }
  }

  static setOnceLocationLatest(isEnable) {
    if (Platform.OS === 'android') {
      RNGeoLocationAMap.setOnceLocationLatest(isEnable);
    }
  }

  static setWifiScan(isEnable) {
    if (Platform.OS === 'android') {
      RNGeoLocationAMap.setWifiScan(isEnable);
    }
  }

  static calculateLineDistance(lat1, lgt1, lat2, lgt2) {
    if (Platform.OS === 'android') {
      return RNGeoLocationAMap.calculateLineDistance(lat1, lgt1, lat2, lgt2);
    }
  }

  static addEventListener(eventType, callback) {
    return GeoLocationAMapEventEmitter.addListener(eventType, (data) => callback(data));
  }

  static getCurrentPosition(success, error, options) {
    this.start();
    const subscription = this.addEventListener('onLocationChanged', (result) => {
      locationChangedHandler(result, success, error);
      this.stop();
      subscription.remove();
    });
  }

  static watchPosition(success, error, options) {
    this.start();
    const subscription = this.addEventListener('onLocationChanged', (result) => {
      locationChangedHandler(result, success, error)
    });
    this._subscriptions.set(++watchId, subscription);
    return watchId;
  }

  static clearWatch(watchId) {
    if (watchId) {
      const subscription = this._subscriptions.get(watchId);
      this._subscriptions.delete(watchId);
      subscription && subscription.remove();
    }
    if (this._subscriptions.size === 0) {
      this.stop();
    }
  }
}
/**
 * 定位场景
 */
export const GeoLocationAMapPurpose = {
  SignIn: 'SignIn',
  Sport: 'Sport',
  Transport: 'Transport',
};
/**
 * 语言
 */
export const GeoLocationLanguage = {
  DEFAULT: 'DEFAULT',
  CN: 'CN',
  EN: 'EN',
};

export default GeoLocationAMap;
